import dbus
from PyQt4 import QtCore, QtGui, uic

from . import __path__
from core.auimd_widget import *

from startup import Startup
from general import General
from call_ui import CallUi
from dialer import Dialer
from pin_dialog import PinDialog
from keyboard import Keyboard

class Call(object):
  INCOMING=0
  OUTGOING=1
  def __init__(self, id, peer, type):
    self.id=id
    self.peer=peer
    self.type=type

class Phone(QtGui.QStackedWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/icons/voicecall.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Phone"

  @staticmethod
  def getCategory():
    return "Network"

  @staticmethod
  def monoInstance():
    return True

  def __init__(self, parent, *args):
    QtGui.QStackedWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    # dict of id -> Call and id -> CallUi
    self.calls={}
    self.call_uis={}

    # init UI components
    self.startup=Startup(self)
    self.general=General(self)
    self.addWidget(self.startup)
    self.addWidget(self.general)
    self.setCurrentWidget(self.startup)

    self.dialer=Dialer(self.general.tab_dialer)
    self.connect(self.dialer, QtCore.SIGNAL("callPeer"), self.initiateOutgoingCall)
    l=QtGui.QVBoxLayout(self.general.tab_dialer)
    l.addWidget(self.dialer)
    l.setContentsMargins(0, 0, 0, 0)

    QtGui.QVBoxLayout(self.general.tab_call_log).addWidget(QtGui.QTextEdit("Hey ! This is at most alpha stage, you won't think that everything is ready, will you ?", self.general.tab_call_log))
    QtGui.QVBoxLayout(self.general.tab_contacts).addWidget(QtGui.QTextEdit("Go back to the dialer tab, and use your own memory to remember phone numbers !", self.general.tab_contacts))
     
    self.right_button.setText("Close")

    # init DBus components
    try:
      self.system_bus=dbus.SystemBus()
      
      self.usage=self.system_bus.get_object("org.freesmartphone.ousaged", "/org/freesmartphone/Usage")
      self.usage_iface=dbus.Interface(self.usage, "org.freesmartphone.GSM.Device")

      self.gsm_device=self.system_bus.get_object("org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device")
      self.gsm_device_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Device")
      self.gsm_network_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Network")
      self.gsm_call_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.Call")
      self.gsm_sim_iface=dbus.Interface(self.gsm_device, "org.freesmartphone.GSM.SIM")

      self.gsm_call_iface.connect_to_signal("CallStatus", self.onCallStatus)
 
      self.init=self.initiator()
      self.init.next()
      #self.setCurrentWidget(self.general)
    except:
      print "dbus init failed"

  def initiator(self):
    # first request the resource
    self.displayInitMessage("Initializing GSM resources")
    self.usage.RequestResource("GSM", reply_handler=self.onRequestResourceReply, error_handler=self.onRequestResourceError)
    yield

    # antenna power up
    self.displayInitMessage("Powering up antenna")
    self.gsm_device_iface.SetAntennaPower(1, reply_handler=self.onSetAntennaPowerReply, error_handler=self.onSetAntennaPowerError)
    yield

    if not self.antenna_ok:
      # check whether a PIN is needed
      self.displayInitMessage("Does the SIM require a PIN code")
      self.gsm_sim_iface.GetAuthStatus(reply_handler=self.onGetAuthStatusReply, error_handler=self.onGetAuthStatusError)
      yield

      # request the user to enter the PIN code if needed
      if self.auth_status=="SIM PIN":
        self.displayInitMessage("Requesting PIN code")
        #(pin_code, ok)=QtGui.QInputDialog.getText(self, "Enter pin code", "pin code", QtGui.QLineEdit.Normal, "1967")
        pin_dialog=PinDialog()
        res=self.desktop.dialog(self, pin_dialog, "PIN code")

        if res==QtGui.QDialog.Accepted and pin_dialog.pin_code:
          # if a pin code has been entered, check it
          self.displayInitMessage("Validating PIN code")
          self.gsm_sim_iface.SendAuthCode(str(pin_dialog.pin_code), reply_handler=self.onSendAuthCodeReply, error_handler=self.onSendAuthCodeError)
          yield
          if self.auth_status!="READY":
            self.initStepFailed("Wrong PIN code")
        else:
          self.initStepFailed("No PIN given")

    # register to home network
    self.displayInitMessage("Registering to the network")
    self.gsm_network_iface.Register(reply_handler=self.onRegisterReply, error_handler=self.onRegisterError, timeout=300)
    yield

  def displayInitMessage(self, msg):
    self.startup.displayInitMessage(msg)

  def initStepSuccessful(self, msg):
    self.startup.initStepSuccessful(msg)
    try:
      self.init.next()
    except StopIteration:
      self.setCurrentWidget(self.general)
    

  def initStepFailed(self, msg):
    self.startup.initStepFailed(msg)
    del self.init

  def onRequestResourceReply(self):
    self.initStepSuccessful("ok")

  def onRequestResourceError(self, e):
    self.initStepFailed("failed")

  def onSetAntennaPowerReply(self):
    self.antenna_ok=True
    self.initStepSuccessful("ok")

  def onSetAntennaPowerError(self, e):
    self.antenna_ok=False
    self.initStepSuccessful("checking auth status")

  def onGetAuthStatusReply(self, auth_status):
    self.auth_status=auth_status
    self.initStepSuccessful(auth_status)

  def onGetAuthStatusError(self, e):
    self.initStepFailed("error")

  def onSendAuthCodeReply(self):
    self.auth_status="READY"
    self.initStepSuccessful("accepted")
    
  def onSendAuthCodeError(self, e):
    self.initStepFailed("rejected")

  def onRegisterReply(self):
    self.initStepSuccessful("ok")
 
  def onRegisterError(self, e):
    self.initStepFailed("failed")

  def rightButtonAction(self):
    self.desktop.closeWidget(self)


  def minimumSizeHint(self):
      return QtCore.QSize(0, 0)


  def onCallStatus(self, id, status, properties):
    if status=="incoming":
      self.desktop.showWidget(self)
      if id not in self.calls:
        try:
          peer=properties["peer"]
        except:
          peer="unknown"

        call=Call(id, peer, Call.INCOMING)
        call_ui=CallUi(self, call)
        self.calls[id]=call
        self.call_uis[id]=call_ui
        self.connect(call_ui, QtCore.SIGNAL("callAccepted"), self.onCallAccepted)
        self.connect(call_ui, QtCore.SIGNAL("callEnded"), self.onCallEnded)
        self.addWidget(call_ui)
        self.setCurrentWidget(call_ui)

    elif status=="outgoing":
      call_ui=CallUi(self, self.calls[id])
      self.call_uis[id]=call_ui
      self.connect(call_ui, QtCore.SIGNAL("callEnded"), self.onCallEnded)
      self.addWidget(call_ui)
      self.setCurrentWidget(call_ui)

    elif status=="active":
      call_ui=self.call_uis[id]
      call_ui.setStatus(status)
    elif status=="release":
      self.setCurrentWidget(self.general)
      self.removeWidget(self.call_uis[id])
      del self.calls[id]
      del self.call_uis[id]

  def initiateOutgoingCall(self, peer):
    id=self.gsm_call_iface.Initiate(str(peer), "voice")
    call=Call(id, peer, Call.OUTGOING)
    self.calls[id]=call

  def onCallAccepted(self, call):
    self.gsm_call_iface.Activate(call.id)

  def onCallEnded(self, call):
    self.gsm_call_iface.Release(call.id)
