#!/usr/bin/env python

import sys,os

from PyQt4 import QtCore, QtGui

import dbus
import dbus.mainloop.qt

from xml.dom.minidom import parseString

from . import __path__
from core.auimd_widget import *

class DBusMethodArgument(object):
  def __init__(self, name, type):
    self.name=name
    self.type=type

class DBusMethodCallDialog(QtGui.QDialog):
  def __init__(self, item):
    QtGui.QDialog.__init__(self)

    # retrieve method data
    self.bus=item.treeWidget().bus
    self.bus_name=item.treeWidget().bus_name
    self.path=item.node_path
    self.method_name=str(item.text(0))
    self.method_iface=str(item.parent().parent().text(0))

    # the dialog first shows a list of arguments, then the result
    sl=QtGui.QStackedLayout(self)
    self.arguments=QtGui.QWidget(self)
    self.result=QtGui.QWidget(self)
    sl.addWidget(self.arguments)
    sl.addWidget(self.result)

    # build the arguments widget
    self.values=[]
    self.signature=""
    vl=QtGui.QVBoxLayout(self.arguments)
    vl.addWidget(QtGui.QLabel("not fully implemented, use with care..."))
    for arg in item.args:
      l=QtGui.QHBoxLayout()
      if not arg.name:
        arg.name="<no name>"
      self.signature+=arg.type
      l.addWidget(QtGui.QLabel(arg.name+" ("+arg.type+")", self.arguments))
      val=QtGui.QLineEdit(self.arguments)
      l.addWidget(val)
      self.values.append(val)
      vl.addLayout(l)

    call=QtGui.QPushButton("Call", self.arguments)
    self.connect(call, QtCore.SIGNAL("clicked()"), self.onCallClicked)
    vl.addWidget(call)

    # build the result widget
    vl=QtGui.QVBoxLayout(self.result)
    vl.addWidget(QtGui.QLabel("Result:", self.result))
    self.result_text=QtGui.QTextEdit(self.result)
    vl.addWidget(self.result_text)
    ok=QtGui.QPushButton("Ok", self.result)
    vl.addWidget(ok)
    self.connect(ok, QtCore.SIGNAL("clicked()"), self.onOkClicked)

  def onCallClicked(self):
    args=map(lambda x: str(x.text()), self.values)
    reply=self.bus.call_blocking(self.bus_name, self.path, self.method_iface, self.method_name, self.signature, args)
    self.result_text.setText(str(reply))
    self.layout().setCurrentWidget(self.result)

  def onOkClicked(self):
    self.accept()

class DBusTreeView(QtGui.QTreeWidget):
  ITEM_NODE=QtGui.QTreeWidgetItem.UserType
  ITEM_IFACE=QtGui.QTreeWidgetItem.UserType+1
  ITEM_METHOD_CONT=QtGui.QTreeWidgetItem.UserType+2
  ITEM_METHOD=QtGui.QTreeWidgetItem.UserType+3
  ITEM_SIGNAL_CONT=QtGui.QTreeWidgetItem.UserType+4
  ITEM_SIGNAL=QtGui.QTreeWidgetItem.UserType+5

  def __init__(self, widget, bus, bus_name):
    QtGui.QTreeWidget.__init__(self)
    self.widget=widget
    self.bus=bus
    self.bus_name=bus_name
  
    root=QtGui.QTreeWidgetItem(self, ['/'], self.ITEM_NODE)

    self.introspect(root, '/')

    self.connect(self, QtCore.SIGNAL("itemExpanded(QTreeWidgetItem*)"), self.onItemExpanded)
    self.connect(self, QtCore.SIGNAL("itemClicked(QTreeWidgetItem*,int)"), self.onItemClicked)

  def onItemExpanded(self, item):
    # introspect only on expand, in order to avoid a heavy recursive scan
    if item.type()==self.ITEM_NODE:
      if not item.populated:
        self.introspect(item, item.path)

  def onItemClicked(self, item, column):
    if item.type()==self.ITEM_METHOD:
      d=DBusMethodCallDialog(item)
      self.widget.desktop.dialog(self.widget, d, "Method call", "", "")

  def introspect(self, item, path):
    reply=self.bus.call_blocking(self.bus_name, path, "org.freedesktop.DBus.Introspectable", "Introspect", "", [])
    tree=parseString(reply)

    for i in [ i for i in tree.documentElement.childNodes if i.nodeType==i.ELEMENT_NODE and i.localName=="interface"]:
      iface_item=QtGui.QTreeWidgetItem([i.getAttribute("name")], self.ITEM_IFACE)
      item.addChild(iface_item)
      methods_item=None
      signals_item=None

      for m in [ m for m in i.childNodes if m.nodeType==m.ELEMENT_NODE and m.localName=="method"]:
        if not methods_item:
          methods_item=QtGui.QTreeWidgetItem(["Methods"], self.ITEM_METHOD_CONT)
          iface_item.addChild(methods_item)
        method_item=QtGui.QTreeWidgetItem([m.getAttribute("name")], self.ITEM_METHOD)
        method_item.node_path=path
        method_item.args=[]
        for a in [ a for a in m.childNodes if a.nodeType==m.ELEMENT_NODE and a.localName=="arg" and a.getAttribute("direction")=="in"]:
          method_item.args.append(DBusMethodArgument(a.getAttribute("name"), a.getAttribute("type")))
        methods_item.addChild(method_item)

      for s in [ s for s in i.childNodes if s.nodeType==s.ELEMENT_NODE and s.localName=="signal"]:
        if not signals_item:
          signals_item=QtGui.QTreeWidgetItem(["Signals"], self.ITEM_SIGNAL_CONT)
          iface_item.addChild(signals_item)
        signal_item=QtGui.QTreeWidgetItem([s.getAttribute("name")], self.ITEM_SIGNAL)
        signals_item.addChild(signal_item)

    for n in [ n for n in tree.documentElement.childNodes if n.nodeType==n.ELEMENT_NODE and n.localName=="node"]:
      child=n.getAttribute("name")
      if path=='/':
        child_path='/'+child
      else:
        child_path=path+'/'+child
      node_item=QtGui.QTreeWidgetItem([child], self.ITEM_NODE)
      node_item.setChildIndicatorPolicy(QtGui.QTreeWidgetItem.ShowIndicator)
      node_item.path=child_path
      node_item.populated=False
      item.addChild(node_item)

    item.populated=True

class DBusBrowser(QtGui.QTabWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetTitle():
    return "DBus Browser"

  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/view_tree.png")

  @staticmethod
  def getCategory():
    return "Tools"

  def __init__(self, parent):
    QtGui.QTabWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    self.system_names=QtGui.QListWidget(self)
    self.session_names=QtGui.QListWidget(self)

    try:
      self.system_bus=dbus.SystemBus()
      reply=self.system_bus.call_blocking('org.freedesktop.DBus', '/', 'org.freedesktop.DBus', 'ListNames', '', [])
      self.system_names.addItems([n for n in reply if n[0]!=':'])
    except:
      pass

    try:
      self.sesssion_bus=dbus.SessionBus()
      reply=self.session_bus.call_blocking('org.freedesktop.DBus', '/', 'org.freedesktop.DBus', 'ListNames', '', [])
      self.session_names.addItems([n for n in reply if n[0]!=':'])
    except:
      pass


    self.addTab(self.system_names, "System")
    self.addTab(self.session_names, "Session")

    self.connect(self.system_names, QtCore.SIGNAL("itemClicked(QListWidgetItem*)"), self.onSystemItemClicked)
    self.connect(self.session_names, QtCore.SIGNAL("itemClicked(QListWidgetItem*)"), self.onSessionItemClicked)

    self.right_button.setText("Close")

  def rightButtonAction(self):
    self.desktop.closeWidget(self)

  def onSystemItemClicked(self, item):
    self.onItemClicked(item, self.system_bus)

  def onSessionItemClicked(self, item):
    self.onItemClicked( item, self.session_bus)

  def onItemClicked(self, item, bus):
    name=item.text()
    for i in range(self.count()):
      if self.tabText(i)==name:
        self.setCurrentIndex(i)
        return

    w=DBusTreeView(self, bus, str(name))
    self.setCurrentIndex(self.addTab(w, name))
